"""Tests related to the --dir command line option."""

import filecmp
from pathlib import Path

from cwlupgrader.main import main

from .util import get_data, get_path


def test_draft3_workflow(tmp_path: Path) -> None:
    """Confirm that --dir works when the directory doesn't exist yet."""
    out_dir = tmp_path / "new"
    main([f"--dir={out_dir}", "--v1-only", get_data("testdata/draft-3/wf.cwl")])
    result = filecmp.cmp(
        get_path("testdata/v1.0/wf.cwl"),
        out_dir / "wf.cwl",
        shallow=False,
    )
    assert result



"""
Example configuration for pytest + cwltest plugin using cwltool directly.

Calls cwltool via Python, instead of a subprocess via `--cwl-runner cwltool`.
"""

import json
from io import StringIO
from typing import Any, Dict, List, Optional, Tuple

from cwltest import utils


def pytest_cwl_execute_test(
    config: utils.CWLTestConfig, processfile: str, jobfile: Optional[str]
) -> Tuple[int, Optional[Dict[str, Any]]]:
    """Use the CWL reference runner (cwltool) to execute tests."""
    from cwltool import main
    from cwltool.errors import WorkflowException

    stdout = StringIO()
    argsl: List[str] = [f"--outdir={config.outdir}"]
    if config.runner_quiet:
        argsl.append("--quiet")
    elif config.verbose:
        argsl.append("--debug")
    argsl.extend(config.args)
    argsl.append(processfile)
    if jobfile:
        argsl.append(jobfile)
    try:
        result = main.main(argsl=argsl, stdout=stdout)
    except WorkflowException:
        return 1, {}
    out = stdout.getvalue()
    return result, json.loads(out) if out else {}



"""Shared test utility functions."""

import atexit
import os
from contextlib import ExitStack
from importlib.resources import as_file, files
from pathlib import Path


def get_path(filename: str) -> Path:
    """Get the file Path for a given test file."""
    # normalizing path depending on OS or else it will cause problem when joining path
    filename = os.path.normpath(filename)
    filepath = None
    try:
        file_manager = ExitStack()
        atexit.register(file_manager.close)
        traversable = files("cwlupgrader") / filename
        filepath = file_manager.enter_context(as_file(traversable))
    except ModuleNotFoundError:
        pass
    if not filepath or not filepath.is_file():
        filepath = Path(os.path.dirname(__file__), os.pardir, filename)
    return filepath.resolve()


def get_data(filename: str) -> str:
    """Get the filename as string for a given test file."""
    return str(get_path(filename))
